;==============================================================================
;
; MSX スターソルジャー
;
;	ADXドライバ呼び出し
;
;==============================================================================
adxdrv_org		equ	#8000
ADX_STARTDRV	EQU	8000H	;DRIVER START
ADX_STOPDRV		EQU	8199H	;DRIVER STOP
ADX_ADXDRV		EQU	81EFH	;DRIVER ADRESS

ADX_WM0			EQU 911BH	;DRIVER SW   0:OFF  1:ON
ADX_WM2			EQU 911DH	;EFFECT CH1  0:OFF  1:PLAY  2-n:Nunber
ADX_WM3			EQU 911EH	;EFFECT CH2  0:OFF  1:PLAY  2-n:Nunber
ADX_WM4			EQU 911FH	;MASTER VOLUME  0~15

ADX_WM15		EQU 912AH	;INTERVAL COUNT
ADX_WM28		EQU 912FH	;ADX DATA ADRESS
ADX_WM31		EQU 9132H	;CHAPTER Nunber
ADX_WM47		EQU 9138H	;REPEAT COUNTER      255:END MUSIC STOP DRV
ADX_WM53		EQU 913BH	;DRV CPU MODE  0  Z80 MODE  /  1  R800 MODE




;===============================================================
; ADXDRV 周期処理呼び出し
;===============================================================
ADXIF_scan:
		call	ADXIF_enter_slot
		call	adxdrv_scan
		jp		ADXIF_leave_slot

;===============================================================
; ADXDRV ドライバロード
;===============================================================
ADXIF_setup:
proc
		call	ADXIF_enter_slot

		ld		hl,adxdrv_src
		ld		de,adxdrv_org
		ld		bc,adxdrv_end-adxdrv_org
		ldir

		ld		a,(#002d)
		cp		#03
		sbc		a,a
		inc		a
		ld		(ADX_WM53),a	; CPU MODE

		ld		a,def_adx_volume
		ld		(ADX_WM4),a		; マスターボリューム

		jp		ADXIF_leave_slot
endp



;===============================================================
; RAMスロット切り替え
;===============================================================
ADXIF_enter_slot:
		ld		a,(ram_slot_8000)
		ld		h,#80
		jp		#0024

ADXIF_leave_slot:
		ld		a,(my_slot)
		ld		h,#80
		jp		#0024



;==============================================================================
;
;	ADXDRV本体
;
;==============================================================================
adxdrv_src:
org		adxdrv_org
		incbin	"adx/ADXDRV.BIN"
adxdrv_scan:
proc
		; BGMのバンクに切り替え
		ld		hl,(current_bank_4000)
		push	hl
		ld		a,bank_adxdat
		call	set_bank_4000
		ld		a,bank_adxdat+1
		call	set_bank_6000

		; ポーズ中
		ld		a,(#ce82)
	_play_or_mute:
		jp		_play
	_play:
		or		a
		jr		nz,_to_mute

		; SEセット
		call	_set_se

		; BGMセット
	_check_bgm:
		ld		a,(ADX_WM47)	;CHECK REPEAT & END
		inc		a
		ld		hl,#ce95
		jr		z,_stop_drv
		ld		a,(hl)
		or		a
		jr		nz,_check_bgm_req
	_stop_drv:
		ld		a,(hl)
		cp		#8a				;自機撃破BGM:10は演奏終了しても#ce95をクリアしない
		jr		z,_keep_silent
		xor		a
	_keep_silent:
		ld		(hl),a
		call	ADX_STOPDRV
		jp		_scan

	_check_bgm_req:
		rla
		jr		c,_scan
		set		7,(hl)
		add		a,+(adxdat-2)
		ld		l,a
		ld		h,adxdat>>8	;#4000-2のため、繰り上がり前提。マイナスしない
		ld		e,(hl)
		inc		l
		ld		d,(hl)
		push	de
		call	ADX_STOPDRV
		pop		hl
		ld		(ADX_WM28),hl	;ADX ADDRESS
		ld		a,def_adx_volume
		ld		(ADX_WM4),a		; マスターボリューム
		xor		a
		ld		(ADX_WM31),a	;CHAPTER Number
		call	ADX_STARTDRV

		; 演奏周期処理
	_scan:
		call	ADX_ADXDRV

		; 処理処理
	_ret:
		pop		hl
		ld		a,l
		call	set_bank_4000
		ld		a,h
		jp		set_bank_6000

		; ポーズ中の処理
	_to_mute:
		ld		hl,_mute
		ld		(_play_or_mute+1),hl
		call	ADX_STOPDRV
		jp		_ret
	_mute:
		or		a
		jp		nz,_ret
		ld		hl,_play
		ld		(_play_or_mute+1),hl
		ld		a,def_adx_volume
		ld		(ADX_WM4),a		; マスターボリューム
		call	ADX_STARTDRV
		jp		_ret

if 0
		; SEセット
	_set_se:
		ld		hl,#ce88
		ld		de,ADX_WM3
		call	_set_se_core
		inc		l
		dec		e
	_set_se_core:
		ld		a,(de)
		or		a
		ret		nz
		ld		a,(hl)
		or		a
		ret		z
		ld		(hl),0
	_do_set_se:
		add		a,  _se-1
		ld		b,+(_se-1)>>8
		ld		c,a
		ld		a,(bc)
		ld		(de),a
		ret
	_se:	;SE番号
		db	1+2		; 1	シールド減
		db	2+2		; 2	跳ね返し(2,9共用)  
		db	0
		db	3+2		; 4	EXPLOSION GROUND
		db	8+2		; 5	EXPLOSION AIRBONE
		db	0
		db	4+2		; 7	SHOT
		db	0
		db	2+2		; 9 ラザロ合体音(2,9共用)
		db	5+2		;10 ボス逃亡
		db	5+2		;11	ITEM/WARP
		db	6+2		;12	BONUS
		db	0+2		;	MIA
		db	7+2		; 	EXTEND	;FC版にある追加した音。
else
		; SEセット
	_set_se:
		ld		hl,#ce88
		ld		de,ADX_WM3
		call	_set_se_core
		inc		l
		dec		e
	_set_se_core:
		ld		a,(hl)
		or		a
		ret		z
		rla
		jr		nc,_do_set_se
		sub		2
		jr		z,_end_set_se
		rrca
		or		#80
		jr		_end_set_se
	_do_set_se:
		add		a,+(_se-2)
		ld		b,+(_se-2)>>8
		ld		c,a
		ld		a,(bc)
		ld		(de),a
		inc		c
		ld		a,(bc)
	_end_set_se:
		ld		(hl),a
		ret

	_se:	;SE番号,再生中ビット+優先時間
		db	1+2,#84		; 1	シールド減
		db	2+2,#84		; 2	跳ね返し(2,9共用)  
		db	0,0
		db	3+2,#bf		; 4	EXPLOSION GROUND
		db	8+2,#bf		; 5	EXPLOSION AIRBONE
		db	0,0
		db	4+2,0		; 7	SHOT
		db	0,0
		db	2+2,#84		; 9 ラザロ合体音(2,9共用)
		db	5+2,#df		;10 ボス逃亡
		db	5+2,#df		;11	ITEM/WARP
		db	6+2,#df		;12	BONUS
		db	0+2,0		;	MIA
		db	7+2,#df		; 	EXTEND	;FC版にある追加した音。
endif
endp


adxdrv_end:
org		adxdrv_src + adxdrv_end-adxdrv_org
