;==============================================================================
;
; MSX スターソルジャー
;
;	ADXドライバ呼び出し
;
;==============================================================================
adxdrv_org		equ	#8000
ADX_STARTDRV	EQU	8000H	;DRIVER START
ADX_STOPDRV		EQU	8199H	;DRIVER STOP
ADX_ADXDRV		EQU	81EFH	;DRIVER ADRESS

ADX_WM_org		EQU 90EEH

ADX_WM0			EQU ADX_WM_org		;DRIVER SW   0:OFF  1:ON
ADX_WM2			EQU ADX_WM_org+2	;EFFECT CH1  0:OFF  1:PLAY  2-n:Nunber
ADX_WM3			EQU ADX_WM_org+3	;EFFECT CH2  0:OFF  1:PLAY  2-n:Nunber
ADX_WM4			EQU ADX_WM_org+4	;MASTER VOLUME  0~15

ADX_WM15		EQU ADX_WM_org+#0e	;INTERVAL COUNT
ADX_WM28		EQU ADX_WM_org+#14	;ADX DATA ADRESS
ADX_WM31		EQU ADX_WM_org+#17	;CHAPTER Nunber
ADX_WM47		EQU ADX_WM_org+#1d	;REPEAT COUNTER	255:END MUSIC STOP DRV
ADX_WM53		EQU ADX_WM_org+#20	;DRV CPU MODE	0  Z80 MODE  /  1  R800 MODE

ADXIF_volume	equ	15


;===============================================================
; ADXDRV 周期処理呼び出し
;===============================================================
ADXIF_scan:
		call	ADXIF_enter_slot
		call	adxdrv_scan
		jp		ADXIF_leave_slot

;===============================================================
; ADXDRV ドライバロード
;===============================================================
ADXIF_setup:
proc
		call	ADXIF_enter_slot

		ld		hl,adxdrv_src
		ld		de,adxdrv_org
		ld		bc,adxdrv_end-adxdrv_org
		ldir

		ld		a,(#002d)
		cp		#03
		sbc		a,a
		inc		a
		ld		(ADX_WM53),a	; CPU MODE

		ld		a,ADXIF_volume
		ld		(ADX_WM4),a		; マスターボリューム

		jp		ADXIF_leave_slot
endp



;===============================================================
; RAMスロット切り替え
;===============================================================
ADXIF_enter_slot:
		ld		a,(ram_slot_8000)
		ld		h,#80
		jp		#0024

ADXIF_leave_slot:
		ld		a,(my_slot)
		ld		h,#80
		jp		#0024



;==============================================================================
;
;	ADXDRV本体
;
;==============================================================================
adxdrv_src:
org		adxdrv_org
		incbin	"adx/ADXDRV.BIN"

; adxdrv呼び出し
adxdrv_scan:
proc
		; BGMのバンクに切り替え
		ld		hl,(current_bank_4000)
		push	hl
		ld		a,bank_adxdat
		call	set_bank_4000
		ld		a,bank_adxdat+1
		call	set_bank_6000

		; ポーズ中?
		ld		a,(#ce82)
	_play_or_pause:
		jp		_play			; 自己書き換え
	_play:
		or		a
		jp		nz,_to_pause


		; イントロカウンタ
		ld		a,(#cf19)
		and		7
		jr		nz,_end_intro_counter
	_intro_counter:
		ld		a,0				; 自己書き換え
		or		a
		jr		z,_end_intro_counter
		dec		a
		ld		(_intro_counter+1),a
		jr		nz,_end_intro_counter
		; 敵出現ウェイト解除のため、イントロ終了後に曲IDだけを変える
	_intro_change_val:
		ld		a,0				; 自己書き換え
		ld		(#ce95),a
	_end_intro_counter:

		; BGMセット(SE更新の前にやること)
	_check_bgm:
		ld		a,(ADX_WM47)	;CHECK REPEAT & END
		inc		a
		ld		hl,#ce95
		jr		z,_stop_drv
		ld		a,(hl)
		or		a
		jr		nz,_check_bgm_req
	_stop_drv:
		ld		a,(hl)
		cp		#8a				;自機撃破BGM:10は演奏終了しても#ce95をクリアしない
		jr		z,_keep_silent
		xor		a
	_keep_silent:
		ld		(hl),a
		call	ADX_STOPDRV
		jp		_set_bgm_end

	_check_bgm_req:
		rla
		jr		c,_set_bgm_end
		set		7,(hl)
		add		a,a
		add		a,+(adxdat-4)
		ld		l,a
		ld		h,adxdat>>8	;#4000-4のため、繰り上がり前提。マイナスしない
		push	hl
		call	ADX_STOPDRV
		pop		hl
		ld		de,ADX_WM28		;ADX ADDRESS
		ldi
		ldi
		ld		a,(hl)						; イントロカウンタ
		ld		(_intro_counter+1),a
		inc		l
		or		a
		ld		a,(hl)
		jr		z,_set_chapter
		ld		(_intro_change_val+1),a		; イントロ終了後に曲IDだけを変える
		xor		a
	_set_chapter:
		ld		(ADX_WM31),a	;CHAPTER Number
		ld		a,ADXIF_volume
		ld		(ADX_WM4),a		; マスターボリューム
		call	ADX_STARTDRV
	_set_bgm_end:


		; SEセット(BGM更新の後にやること)
		ld		a,(ADX_WM2)
		or		a
		jr		nz,_end_se_wait
		ld		(_se_priority+1),a
	_end_se_wait:
		ld		hl,#ce88
		ld		b,2
	_lp_set_se:
		ld		a,(hl)
		or		a
		jr		z,_se_next
		ld		(hl),0
		add		a,a
		add		a,+(_se_tbl-2)
		ld		d,+(_se_tbl-2)>>8
		ld		e,a
		ld		a,(de)
	_se_priority:
		cp		0				; 自己書き換え
		jr		c,_se_next
		ld		(_se_priority+1),a
		inc		e
		ld		a,(de)
		ld		(ADX_WM2),a
	_se_next:
		inc		l
		djnz	_lp_set_se


		; 演奏周期処理
	_scan:
		ei
		call	ADX_ADXDRV

		; 処理処理
	_ret:
		; バンク戻して終了
		pop		hl
		ld		a,l
		call	set_bank_4000
		ld		a,h
		jp		set_bank_6000


		; ポーズ中の処理
	_to_pause:
		ld		hl,_pause
		ld		(_play_or_pause+1),hl
		xor		a
		ld		(ADX_WM4),a		; マスターボリューム0
		call	ADX_ADXDRV
		xor		a
		ld		(ADX_WM0),a		; DRIVER SW OFF
		jp		_ret
	_pause:
		or		a
		jp		nz,_ret
		ld		hl,_play
		ld		(_play_or_pause+1),hl
		ld		a,1
		ld		(ADX_WM0),a		; DRIVER SW ON
		ld		a,ADXIF_volume
		ld		(ADX_WM4),a		; マスターボリューム
		jp		_scan


	_se_tbl:	;優先lv, SE番号
		db		3, 1+2		; 1	シールド減
		db		0, 2+2		; 2	跳ね返し(2,9共用)  
		db		0, 0
		db		1, 8+2		; 4	EXPLOSION GROUND
		db		1, 3+2		; 5	EXPLOSION AIRBONE
		db		0, 0
		db		0, 4+2		; 7	SHOT
		db		0, 0
		db		3, 9+2		; 9 ラザロ合体音
		db		2, 0+2		;10 ボス逃亡
		db		4, 5+2		;11	ITEM/WARP
		db		4, 6+2		;12	BONUS
		db		4, 7+2		; 	EXTEND	;FC版にある追加した音。
endp


adxdrv_end:
org		adxdrv_src + adxdrv_end-adxdrv_org
