;==============================================================================
;
; MSX スターソルジャー
;
;==============================================================================

;==============================================================================
;	文字の書き出し	hl:文字数,文字列	de:vram adr offset
;==============================================================================
editor_put_str:
PROC
		ex		de,hl
		ld		bc,#3820
		add		hl,bc
		ex		de,hl
		ld		b,0
		ld		c,(hl)
		inc		hl
		jp		#005C
ENDP
macro	mPutText, tx, adr
	ld		hl,tx
	ld		de,adr
	call	editor_put_str
endm
;==============================================================================
;	HEXの書き出し	hl:vram adr offset	a:val
;==============================================================================
editor_put_number_hex:
proc
		ld		b,a
		ld		de,#3820
		add		hl,de
		call	#0053

		ld		a,b
		rrca
		rrca
		rrca
		rrca
		call	_put_a_hex
		ld		a,b
	_put_a_hex:
		and		#f
		add		a,'0'
		cp		'0'+10
		jr		c,_out
		add		a,7
	_out:
		out		(c),a
		ret
endp



;================================================================
;	キー入力
;================================================================
editor_get_key:
PROC
		ld		a,#7
		call	#0141
		bit		2,a
		jr		nz,_not_esc
		ld		a,#1b
		or		a
		ret
	_not_esc:
		ld		a,#6
		call	#0141
		bit		5,a
		jr		nz,_not_f1
	_trigger:
		ld		a,#20
		or		a
		ret
	_not_f1:

		call	#49f2
		ld		a,(#cdf1)
		or		a
		jr		nz,_trigger
	_stick:
		ld		a,(#cdee)
		inc		a
		rra
		ld		h,_stick_tbl>>8
		add		a,_stick_tbl&#ff
		ld		l,a
		ld		a,(hl)
		or		a
		ret		nz

		xor		a
		ld		l,'0'
		ld		b,8
	_chk_num:
		call	#0141
	_lp_num:
		rrca
		jr		nc,_ret_num
		inc		l
		djnz	_lp_num

		cp		'8'
		jr		nz,_ret_off

		ld		a,1
		ld		l,'8'
		ld		b,2
		jr		_chk_num
	_ret_num:
		ld		a,l
		or		a
		ret

	_ret_off:
		xor		a
		ret


	_stick_tbl:
		db		0,#1e,#1c,#1f,#1d
ENDP

;================================================================
;	編集系パラメータ初期化
;================================================================
init_menu_mode:
PROC
		xor		a
		ld		(cfg_cursor_pos),a

		ld		a,def_trapzone_mode
		ld		(cfg_trapzone_mode),a

		ld		a,(#002d)
		cp		#03
		sbc		a,a
		inc		a
		ld		(cfg_sound_driver),a

		xor		a
		ld		(cfg_bgm_test),a

		ld		a,def_map_type
		ld		(cfg_map_type),a

		ld		hl,def_map_list_adr
		ld		(cfg_map_list_adr),hl
ENDP
;================================================================
;	コンフィグ全適用
;================================================================
apply_all_config:
PROC
		ld		a,(cfg_trapzone_mode)
		or		a
		jr		z,_set_trapzone_mode_end
		xor		a
		ld		(#cf1f),a	; 暗黒クリア
	_set_trapzone_mode_end:

		ld		hl,map_list_msx
		ld		a,(cfg_map_type)
		or		a
		jr		z,_set
		ld		hl,map_list_fc
	_set:
		ld		(cfg_map_list_adr),hl

		jp		switch_sound_driver
ENDP




;================================================================
;	編集モード切り替え
;================================================================
editor_setup:
PROC
		; 画面クリア
		call	editor_cls
		call	editor_key_off_wait

		; キークリック音on
		ld		a,1
		ld		(CLIKSW),a
		ret
ENDP

editor_close:
PROC
		; キークリック音off
		xor		a
		ld		(CLIKSW),a

		call	editor_cls
		call	dumb_loop
		call	editor_key_off_wait
		ld		a,#ff
		ret
ENDP

; 画面クリア
editor_cls:
		xor		a
		ld		hl,#3820
		ld		bc,#03e0
		call	#0056
		ret

editor_key_off_wait:
PROC
		call	editor_get_key
		jr		nz,editor_key_off_wait
		ret
ENDP

dumb_loop:
PROC
		ld		bc,0
	_lp:
		cpi
		cpd
		ret		po
		jr		_lp
ENDP


;================================================================
;	メニューモード
;================================================================
menu_mode:
PROC
		di
		; ゲーム画面更新停止
		ld		a,(#cde4)
		push	af
		xor		a
		ld		(#cde4),a
		ei

		; BGM記憶
		ld		a,(#ce95)
		push	af

		; メニューループ
		call	main_menu

		; BGM再現
		ld		a,(#ce95)
		ld		c,a
		pop		af
		cp		c
		jr		z,_end_load_bgm
		and		#7f
		ld		(#ce95),a
	_end_load_bgm:

		; コンフィグ適用
		call	apply_all_config

		; トラップゾーン表示切り替え
		call	switch_trapzone_bg
		di
		; マップ再読込
		pop		af
		push	af
		or		a
		jr		z,_end_reload_map
		call	proc_48a0+3
		ld		a,(#cdde)
		or		a
		ld		ix,load_map
		ld		iyl,1
		ld		hl,(#cddf)
		push	hl					;現在位置保存
		call	z,call_bank_6000	;BANK1がほしい
		pop		hl					;現在位置復帰
		ld		(#cddf),hl
	_end_reload_map:

		; ゲーム画面更新再開
		pop		af
		ld		(#cde4),a

		; メッセージ再描画
		ld		a,(#ce69)
		or		a
		jr		z,_no_msg
		push	af
		ld		a,#ff
		ld		(#ce69),a
		call	#5817
		pop		af
		ld		(#ce69),a
	_no_msg:
		ei

		ret
ENDP


main_menu:
PROC
		call	editor_setup

		mPutText	tx_menu_0,  6*32+3
		mPutText	tx_menu_1,  8*32+3
		mPutText	tx_menu_2, 10*32+3
		mPutText	tx_menu_3, 12*32+3
		mPutText	tx_menu_4, 14*32+3

		jr		_key_off_wait

	_main_loop:
		call	editor_get_key
		jr		z,_main_loop
		cp		#1b
		jp		z, _exit

		ld		hl,cfg_cursor_pos

		cp		#1e
		jr		nz,_not_up
		ld		a,(hl)
		dec		a
		jp		p,_not_top
		add		a,menu_cnt
	_not_top:
		ld		(hl),a
		jr		_key_off_wait
	_not_up:
		cp		#1f
		jr		nz,_not_down
		ld		a,(hl)
		inc		a
		cp		menu_cnt
		jr		c,_not_bottom
		xor		a
	_not_bottom:
		ld		(hl),a
		jr		_key_off_wait
	_not_down:
		ld		b,1
		cp		#20
		jr		z,_space_down
		cp		#1c
		jr		z,_lr_down
	_not_right:
		ld		b,#ff
		cp		#1d
		jr		z,_lr_down
	_key_off_wait:
		call	_update_disp
		call	editor_key_off_wait
		jr		_main_loop

	_space_down:
		ld		a,(hl)
		or		a
		jr		z,_exit
		cp		3
		jr		z,_exec_bgmtest
		jr		_key_off_wait

	_lr_down:
		ld		a,(hl)
		dec		a
		jr		z,_shift_trapzonecfg
		dec		a
		jr		z,_shift_soundcfg
		dec		a
		jr		z,_shift_bgmtest
		dec		a
		jr		z,_shift_maptypecfg
		jr		_key_off_wait

	_exit:
		jp		editor_close

	_shift_trapzonecfg:
		ld		a,(cfg_trapzone_mode)
	_shift_trapzonecfg_ck
		add		a,b
		and		3
		cp		3
		jr		nc,_shift_trapzonecfg_ck
		ld		(cfg_trapzone_mode),a
		jr		_key_off_wait

	_shift_soundcfg:
		ld		a,(cfg_sound_driver)
		add		a,b
		and		1
		ld		(cfg_sound_driver),a
		call	stop_sound_driver
		jr		_key_off_wait

	_shift_bgmtest:
		ld		a,(cfg_bgm_test)
		add		a,b
		and		#f
		ld		(cfg_bgm_test),a
		jr		_key_off_wait
	_shift_maptypecfg:
		ld		a,(cfg_map_type)
		add		a,b
		and		1
		ld		(cfg_map_type),a
		jr		_key_off_wait
	_exec_bgmtest:
		call	stop_sound_driver
		ld		a,(cfg_bgm_test)
		ld		(#ce95),a
		jr		_key_off_wait

	_update_disp:
	; カーソル表示
		ld		hl,14*32+1 + #3820
		ld		b,menu_cnt
	_lp_put_cursor:
		ld		a,(cfg_cursor_pos)
		inc		a
		cp		b
		ld		a,#b6
		jr		z,_put_cursor
		ld		a,0
	_put_cursor:
		call	#004d
		ld		de,-2*32
		add		hl,de
		djnz	_lp_put_cursor

	; 潜る表示
		ld		de,8*32+20
		ld		a,(cfg_trapzone_mode)
		ld		hl,tx_trapzone_mode_0
		or		a
		jr		z,_put_trapzone_mode
		ld		hl,tx_trapzone_mode_1
		dec		a
		jr		z,_put_trapzone_mode
		ld		hl,tx_trapzone_mode_2
	_put_trapzone_mode:
		call	editor_put_str

	; サウンドドライバー表示
		ld		de,10*32+20
		ld		a,(cfg_sound_driver)
		or		a
		ld		hl,tx_sound_driver_0
		jr		z,_put_sound_driver
		ld		hl,tx_sound_driver_1
	_put_sound_driver:
		call	editor_put_str

	; BGMテスト表示
		ld		hl,12*32+20
		ld		a,(cfg_bgm_test)
		call	editor_put_number_hex

	; map type表示
		ld		de,14*32+20
		ld		a,(cfg_map_type)
		or		a
		ld		hl,tx_map_type_0
		jr		z,_put_map_type
		ld		hl,tx_map_type_1
	_put_map_type:
		jp		editor_put_str
ENDP




;==============================================================================
; トラップゾーンの表示/非表示
;==============================================================================
switch_trapzone_bg:
proc
		ld		hl,(#ce7c)
		push	hl

		ld		a,(cfg_trapzone_mode)
		cp		2
		ld		hl,#7c96		; 0が4つ
		jr		nz,_set_pcgno
		ld		hl,_trap_pcg
	_set_pcgno:
		ld		(#ce7c),hl

		ld		de,#cc9d
		ld		c,15
	_lp_y:
		ld		a,(cur_bg_y)
		rrca
		add		a,15
		sub		c
		sub		2
		and		#f
		ld		h,a
		xor		a
		rr		h
		rra
		rr		h
		rra
		ld		l,a

		ld		b,20
	_lp_x:
		ld		a,(de)
		cp		#96
		jr		nz,_next
	_put:
		push	hl
		push	de
		push	bc

		ex		de,hl
		ld		hl,(#ce7c)
		ld		bc,_ret_putchr16x16		; ret point
		push	bc							;
		push	bc			; dummy putchr16x16_でpop deされる
		ld		b,0
		jp		putchr16x16_
	_ret_putchr16x16:
		pop		bc
		pop		de
		pop		hl
	_next:
		inc		l
		inc		de
		djnz	_lp_x
		dec		c
		jr		nz,_lp_y

		pop		hl
		ld		(#ce7c),hl
		ret

	_trap_pcg:
		db		#b6,#b6,#b6,#b6
endp



;==============================================================================
;	文字
;==============================================================================
menu_cnt			equ	5

tx_menu_0:			db	 4,"EXIT"
tx_menu_1:			db	 9,"TRAP",0,"TILE"
tx_menu_2:			db	12,"SOUND",0,"DRIVER"
tx_menu_3:			db	 8,"BGM",0,"TEST"
tx_menu_4:			db	 3,"MAP"

tx_trapzone_mode_0:	db	4,"ON",0,0
tx_trapzone_mode_1:	db	4,"OFF",0
tx_trapzone_mode_2:	db	4,"SHOW"

tx_sound_driver_0:	db	6,"PSG",0,0,0
tx_sound_driver_1:	db	6,"ADXDRV"

tx_map_type_0:		db	3,"MSX"
tx_map_type_1:		db	3,"NES"
